from __future__ import annotations

import os
from collections import defaultdict
from typing import TYPE_CHECKING, Optional, List, Dict, Any, Tuple, Set

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.output.record import SCA_PACKAGE_SCAN_CHECK_NAME, Record
from checkov.common.sast.consts import SastLanguages
from checkov.common.util.type_forcers import convert_str_to_bool
from checkov.sast.report import SastData, SastReport
from checkov.common.sca.consts import get_package_by_str, ScaPackageFile, sca_package_to_sast_lang_map

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner

NORMALIZE_PREFIX = 'BC_NORMALIZE_'


class VulnerabilitiesIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=2)  # must be after the custom policies integration
        # the provider may be lower or upper depending on where the policy was created

    def is_valid(self) -> bool:
        return (
            self.bc_integration.is_integration_configured()
            and not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def pre_scan(self) -> None:
        # not used
        pass

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def post_runner(self, scan_report: Report) -> None:
        # not used
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        self.merge_sca_and_sast_reports(merged_reports)

    def merge_sca_and_sast_reports(self, merged_reports: list[Report]) -> None:
        if not bool(convert_str_to_bool(os.getenv('CKV_ENABLE_UPLOAD_SAST_IMPORTS', False))) or \
                not bool(convert_str_to_bool(os.getenv('CKV_ENABLE_SCA_INTEGRATE_SAST', False))):
            return

        # Extract SAST imports report and reachability report
        sast_reports = [scan_report for scan_report in merged_reports if isinstance(scan_report, SastReport)]
        if not len(sast_reports):
            return

        sast_imports_report = SastData.get_sast_import_report(sast_reports)
        sast_reacability_report = SastData.get_sast_reachability_report(sast_reports)

        # Extract SCA packages report
        sca_packages_report = [scan_report for scan_report in merged_reports if
                               scan_report.check_type == CheckType.SCA_PACKAGE]
        if not len(sca_packages_report):
            return

        # Extract vulnerabilities failed checks
        cves_checks = [cve_check for cve_check in sca_packages_report[0].failed_checks if
                       cve_check.check_name == SCA_PACKAGE_SCAN_CHECK_NAME]

        # Create a dictionary to store the grouped records
        grouped_records = self.group_cves_checks_by_files(cves_checks)

        # Iterate over each file, get sast imports and enrich it's Cves
        for sca_file_path, current_cves in grouped_records.items():

            # Determine the langauge of file
            lang = self.get_sast_lang_by_file_path(sca_file_path)

            # Extract Sast data from Sast report filtered by the language
            imports_entries = sast_imports_report.get('imports', {}).get(lang, {}).items()
            filtered_imports_entries = [(code_file_path, sast_data) for code_file_path, sast_data in imports_entries if
                                        self.is_deeper_or_equal_level(sca_file_path, code_file_path)]

            reachability_entries = sast_reacability_report.get('reachability', {}).get(lang, {}).items()
            filtered_reachability_entries = [(code_file_path, sast_data) for code_file_path, sast_data in
                                             reachability_entries if self.is_deeper_or_equal_level(sca_file_path,
                                                                                                   code_file_path)]

            if not len(filtered_imports_entries) and not len(filtered_reachability_entries):
                continue

            # Create maps with the relevant structure for the enrichment step
            sast_files_by_package_map = self.create_file_by_package_map(filtered_imports_entries)
            sast_reachable_cves_by_package_map = self.create_reachable_cves_by_package_map(filtered_reachability_entries)

            # Enrich the CVEs
            self.enrich_cves_with_sast_data(current_cves, sast_files_by_package_map, sast_reachable_cves_by_package_map)

    '''
    Each SCA report check has file_path, we want to getter same file_path so we won't have to calculate SAST language more then once
    '''

    def group_cves_checks_by_files(self, cves_checks: List[Record]) -> Dict[str, List[Record]]:
        # Create a dictionary to store the grouped records
        grouped_records: Dict[str, List[Record]] = defaultdict()

        # Group the records by the 'file_path' key
        for record in cves_checks:
            file_path = record.file_path
            if file_path not in grouped_records:
                grouped_records[file_path] = list()
            grouped_records[file_path].append(record)

        return grouped_records

    '''
    convert SAST report structure to a sturcture grouped by package_name, for better performance in the enrich step
    '''

    def create_file_by_package_map(self, filtered_entries: List[Tuple[Any, Any]]) -> Dict[str, List[str]]:
        sast_files_by_packages_map: Dict[str, List[str]] = defaultdict(list)
        for code_file_path, sast_data in filtered_entries:
            for package_name in sast_data['all']:
                clean_package_name = package_name.strip("'")

                # in case it is alias-name, getting the real one
                if package_name in sast_data.get('aliases', {}):
                    clean_package_name = sast_data['aliases'][package_name]

                # Normalize package name
                normalize_package_name = self.normalize_package_name(clean_package_name)

                if clean_package_name not in sast_files_by_packages_map:
                    sast_files_by_packages_map[clean_package_name] = list()
                if normalize_package_name not in sast_files_by_packages_map:
                    sast_files_by_packages_map[normalize_package_name] = list()

                sast_files_by_packages_map[clean_package_name].append(code_file_path)
                sast_files_by_packages_map[normalize_package_name].append(code_file_path)

        return sast_files_by_packages_map

    def create_reachable_cves_by_package_map(self, filtered_reachability_entries: List[Tuple[Any, Any]]) -> Dict[str, Set[str]]:
        reachable_cves_by_packages_map: Dict[str, Set[str]] = defaultdict(set)
        for _, file_data in filtered_reachability_entries:
            packages = file_data.packages
            for package_name, package_data in packages.items():
                for function_item in package_data.functions:
                    reachable_cves_by_packages_map[package_name].add(function_item.cve_id)
        return reachable_cves_by_packages_map

#######################################################################################################################
    '''
    enrich each CVE with the risk factor of IsUsed - which means there is a file the use the package of that CVE
    '''

    def _is_package_used_for_cve(self, cve_vulnerability_details: Dict[str, Any], sast_files_by_package_map: Dict[str, List[str]]) -> bool:
        package_name = cve_vulnerability_details.get('package_name', '')
        normalize_package_name = self.normalize_package_name(package_name)
        return package_name in sast_files_by_package_map or normalize_package_name in sast_files_by_package_map

    def _is_reachable_function_for_cve(self, cve_vulnerability_details: Dict[str, Any], sast_reachable_cves_by_package_map: Dict[str, Set[str]]) -> bool:
        package_name = cve_vulnerability_details.get('package_name', '')
        return cve_vulnerability_details.get('id') in sast_reachable_cves_by_package_map.get(package_name, set())

    def enrich_cves_with_sast_data(
            self,
            current_cves: List[Record],
            sast_files_by_package_map: Dict[str, List[str]],
            sast_reachable_cves_by_package_map: Dict[str, Set[str]]
    ) -> None:
        for cve_check in current_cves:
            if cve_check.vulnerability_details:
                is_package_used = self._is_package_used_for_cve(cve_check.vulnerability_details, sast_files_by_package_map)
                cve_check.vulnerability_details.get('risk_factors', {})['IsUsed'] = is_package_used

                is_reachable_function = self._is_reachable_function_for_cve(cve_check.vulnerability_details, sast_reachable_cves_by_package_map)
                cve_check.vulnerability_details.get('risk_factors', {})['ReachableFunction'] = is_reachable_function
#######################################################################################################################

    '''
    we want to consider sast info only on files that are on the same level of the SCA file or deeper.
    '''

    def is_deeper_or_equal_level(self, main_file_path: str, other_file_path: str) -> bool:
        relative_path = os.path.relpath(os.path.dirname(other_file_path), os.path.dirname(main_file_path))
        return not other_file_path.startswith('.') and not relative_path.startswith('..') and not os.path.isabs(
            relative_path)

    '''
    getting file_path of SCA file, like package.json and need to convert it to SAST language like Javascript
    first we are converting the sca file to package file like, and then converting it to SAST language
    '''

    def get_sast_lang_by_file_path(self, file_path: str) -> Optional[SastLanguages]:
        suffix = file_path.split('/').pop() or ''
        sca_package: Optional[ScaPackageFile] = get_package_by_str(suffix)
        if not sca_package:
            return None

        return sca_package_to_sast_lang_map.get(sca_package, None)

    '''
    normalize the package name because there can be different between the package name as it present in the SCA file and
    in the way it used in the code, so we are removing special chars for better comperation
    '''

    def normalize_package_name(self, package_name: str) -> str:
        normalize_package: str = package_name.replace('-', '').replace('_', '')

        if './' in package_name:
            last_index = package_name.rfind('/')
            normalize_package = package_name[last_index + 1:]

        return f"{NORMALIZE_PREFIX}{normalize_package}"


integration = VulnerabilitiesIntegration(bc_integration)



from checkov.arm.checks.resource.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleSSHAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self) -> None:
        super().__init__(
            name="Ensure that SSH access is restricted from the internet", check_id="CKV_AZURE_10", port=22
        )


check = NSGRuleSSHAccessRestricted()



import logging
import os
import json
from typing import Set, List, Dict

from checkov.common.runners.base_runner import IGNORE_HIDDEN_DIRECTORY_ENV, ignored_directories
from checkov.common.sast.consts import SastLanguages


class FilesFilterManager:
    def __init__(self, source_codes: List[str], languages: Set[SastLanguages]) -> None:
        self.source_codes: List[str] = source_codes
        self.languages: Set[SastLanguages] = languages

    def get_files_to_filter(self) -> List[str]:
        files_to_filter: List[str] = []
        try:
            if SastLanguages.JAVASCRIPT in self.languages:
                files_to_filter += self._get_js_files_to_filter()
            files_to_filter += self._filter_hidden_files()
        except Exception as e:
            logging.debug(f'Error filtering js files generated by ts: {e}')
        return files_to_filter

    def _get_js_files_to_filter(self) -> List[str]:
        js_files_to_filter = []

        for path in self.source_codes:
            js_files: List[Dict[str, str]] = []
            ts_files: List[Dict[str, str]] = []
            tsconfig_files: List[Dict[str, str]] = []
            for (dirpath, _, filenames) in os.walk(path):
                if '/node_modules/' in dirpath:
                    continue
                for filename in filenames:
                    if filename.endswith('.ts'):
                        ts_files.append({'full_path': os.sep.join([dirpath, filename]), 'dir': dirpath, 'name': filename})
                    if filename.endswith('tsconfig.json'):
                        tsconfig_files.append({'full_path': os.sep.join([dirpath, filename]), 'dir': dirpath, 'name': filename})
                    if filename.endswith('.js'):
                        js_files.append({'full_path': os.sep.join([dirpath, filename]), 'dir': dirpath, 'name': filename})

            js_files_to_filter += FilesFilterManager._filter_by_tsconfig(tsconfig_files)
            js_files_to_filter += FilesFilterManager._filter_direct_build_js(js_files, ts_files, js_files_to_filter)

        return js_files_to_filter

    @staticmethod
    def _filter_direct_build_js(js_files: List[Dict[str, str]], ts_files: List[Dict[str, str]], filtered_by_tsconfig: List[str]) -> List[str]:
        js_files_to_filter: List[str] = []
        for js_file in js_files:
            js_dir = js_file.get('dir', '')
            already_skipped = False
            for filtered_by_tsconfig_path in filtered_by_tsconfig:
                if js_dir.startswith(filtered_by_tsconfig_path):
                    already_skipped = True
                    break
            if already_skipped:
                continue
            for ts_file in ts_files:
                if ts_file.get('dir', '') == js_dir and ts_file.get('name', '')[:-3] == js_file.get('name', '')[:-3]:
                    js_files_to_filter.append(js_file.get('full_path', ''))
                    break
        return js_files_to_filter

    @staticmethod
    def _filter_by_tsconfig(tsconfig_files: List[Dict[str, str]]) -> List[str]:
        js_files_to_filter: List[str] = []
        for tsconfig_file in tsconfig_files:
            with open(tsconfig_file.get('full_path', '')) as fp:
                config = json.load(fp)
            out_dir = config.get('compilerOptions', {}).get('outDir')
            out_file = config.get('compilerOptions', {}).get('outFile')
            if out_dir:
                build_dir = out_dir
            elif out_file:
                build_dir = out_file
            else:
                build_dir = tsconfig_file.get('dir')

            # relative path
            if not build_dir.startswith('/'):
                build_path = os.path.abspath(tsconfig_file.get('dir', '') + '/' + build_dir)
            # absolute path
            else:
                build_path = build_dir
            js_files_to_filter.append(build_path)
        return js_files_to_filter

    def _filter_hidden_files(self) -> List[str]:
        # consider ENV variable to ignore hidden directories (CKV_IGNORED_DIRECTORIES, CKV_IGNORE_HIDDEN_DIRECTORIES)
        paths_to_filter: List[str] = ignored_directories.copy()
        if not IGNORE_HIDDEN_DIRECTORY_ENV:
            return paths_to_filter
        for path in self.source_codes:
            for (_dirpath, dirs, _) in os.walk(path):
                for directory in dirs:
                    if directory.startswith('.'):
                        paths_to_filter.append(directory)

        return paths_to_filter



import re
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

ENGINE_VERSION_PATTERN = re.compile(r"(\d+\.\d+.\d+)")
ENGINE_VERSION_SHORT_PATTERN = re.compile(r"(\d+\.\d+)")
MINIMUM_ACTIVEMQ_VERSION = 5.17
MINIMUM_RABBITMQ_VERSION = 3.11


class MQBrokerVersion(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure MQ Broker version is current"
        id = "CKV_AWS_208"
        supported_resources = ("aws_mq_broker", "aws_mq_configuration")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    @staticmethod
    def version_string_to_tuple(version_str) -> tuple:
        return tuple(map(int, str(version_str).split('.')))

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("engine_type"):
            mq_type = conf.get("engine_type")[0]
            semantic = conf.get("engine_version", [''])[0]
            if not re.search(ENGINE_VERSION_PATTERN, semantic):
                return CheckResult.UNKNOWN
            version_tuple = self.version_string_to_tuple(re.search(ENGINE_VERSION_SHORT_PATTERN, semantic).group())
            if mq_type in 'ActiveMQ':
                if version_tuple >= self.version_string_to_tuple(MINIMUM_ACTIVEMQ_VERSION):
                    return CheckResult.PASSED

            if mq_type in 'RabbitMQ':
                if version_tuple >= self.version_string_to_tuple(MINIMUM_RABBITMQ_VERSION):
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = MQBrokerVersion()



from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class MSKClusterLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure MSK Cluster logging is enabled"
        id = "CKV_AWS_80"
        supported_resources = ['AWS::MSK::Cluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> Any:
        if 'Properties' in conf.keys():
            if 'LoggingInfo' in conf['Properties'].keys():
                if 'BrokerLogs' in conf['Properties']['LoggingInfo'].keys():
                    logging = conf['Properties']['LoggingInfo']['BrokerLogs']
                    types = ["CloudWatchLogs", "Firehose", "S3"]
                    for x in types:
                        if x in logging and 'Enabled' in logging[x] and logging[x]['Enabled'] is True:
                            return CheckResult.PASSED
        return CheckResult.FAILED


check = MSKClusterLogging()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class KubeletClientCa(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.6 4.2.3
        id = "CKV_K8S_140"
        name = "Ensure that the --client-ca-file argument is set as appropriate"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kubelet" in conf["command"]:
                for command in conf["command"]:
                    if command.startswith("--client-ca-file"):
                        if len(command.split("=")) == 2:
                            if (command.split("=")[1]).strip() != '':
                                return CheckResult.PASSED
                        return CheckResult.FAILED
                return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = KubeletClientCa()



from __future__ import annotations

from typing import TYPE_CHECKING
from abc import ABC
from abc import abstractmethod

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet


class BaseMultiLineParser(ABC):
    def get_lines_from_same_object(
        self,
        search_range: range,
        context: CodeSnippet | None,
        raw_context: CodeSnippet | None,
        line_length_limit: int = 0,
    ) -> set[str]:
        possible_keywords: set[str] = set()

        if not context or not raw_context:
            return possible_keywords
        for j in search_range:
            line = raw_context.lines[j]
            if line_length_limit and len(line) > line_length_limit:
                continue
            if self.consecutive_lines_in_same_object(raw_context=raw_context, other_line_idx=j) \
                    and not self.is_line_comment(line):
                possible_keywords.add(raw_context.lines[j])
                if self.is_object_start(line=line) or self.is_object_end(line=line):
                    return possible_keywords
        # No start of array detected, hence all found possible_keywords are irrelevant
        return set()

    @abstractmethod
    def consecutive_lines_in_same_object(
        self,
        raw_context: CodeSnippet | None,
        other_line_idx: int,
    ) -> bool:
        pass

    @staticmethod
    @abstractmethod
    def is_object_start(
        line: str,
    ) -> bool:
        pass

    @staticmethod
    @abstractmethod
    def is_object_end(
        line: str,
    ) -> bool:
        pass

    @staticmethod
    @abstractmethod
    def is_line_comment(
        line: str
    ) -> bool:
        pass




from __future__ import annotations
from typing import Any, Dict
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class ACRContainerScanEnabled(BaseResourceCheck):
    SKUS = {"Standard", "Premium"}  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        name = "Enable vulnerability scanning for container images."
        id = "CKV_AZURE_163"
        supported_resources = ("Microsoft.ContainerRegistry/registries",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        sku = conf.get("sku", {})
        sku_name = sku.get("name")

        if isinstance(sku_name, str) and sku_name in ACRContainerScanEnabled.SKUS:
            return CheckResult.PASSED

        return CheckResult.FAILED


check = ACRContainerScanEnabled()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ElasticacheReplicationGroupEncryptionAtTransitAuthToken(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in the ElastiCache Replication Group is securely encrypted at transit and has auth token"
        id = "CKV_AWS_31"
        supported_resources = ("AWS::ElastiCache::ReplicationGroup",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        """
            Looks for encryption configuration at aws_elasticache_replication_group:
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html
        :param conf: aws_elasticache_replication_group configuration
        :return: <CheckResult>
        """
        properties = conf. get("Properties")
        if properties and isinstance(properties, dict):
            if "TransitEncryptionEnabled" in properties.keys() and ("AuthToken" in properties.keys() or
                                                                    "UserGroupIds" in properties.keys()):
                if conf["Properties"]["TransitEncryptionEnabled"]:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = ElasticacheReplicationGroupEncryptionAtTransitAuthToken()



from typing import List

from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories


class SagemakerNotebookEncryptedWithCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Sagemaker domain and notebook instance are encrypted by KMS using a customer managed Key (CMK)"
        id = "CKV_AWS_187"
        supported_resources = ("AWS::SageMaker::NotebookInstance", "AWS::SageMaker::Domain")
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/KmsKeyId'

    def get_expected_value(self):
        return ANY_VALUE

    def get_evaluated_keys(self) -> List[str]:
        return ['Properties/KmsKeyId']


check = SagemakerNotebookEncryptedWithCMK()



from typing import List, Any

from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AppServicePythonVersion(BaseResourceValueCheck):

    def __init__(self) -> None:
        name = "Ensure that 'Python version' is the latest, if used to run the web app"
        id = "CKV_AZURE_82"
        supported_resources = ("Microsoft.Web/sites",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.UNKNOWN)

    def get_inspected_key(self) -> str:
        return "properties/siteConfig/pythonVersion"

    def get_expected_values(self) -> List[Any]:
        return ["3.9", "3.10", "3.11", "3.12"]


check = AppServicePythonVersion()



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar, Any  # noqa

from rustworkx import PyDiGraph

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.typing import _RustworkxGraph

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block  # noqa
    from checkov.common.graph.graph_builder.local_graph import LocalGraph

_Block = TypeVar("_Block", bound="Block")


class RustworkxConnector(DBConnector[_RustworkxGraph]):
    def __init__(self) -> None:
        self.graph: _RustworkxGraph = PyDiGraph()

    def save_graph(self, local_graph: LocalGraph[_Block], add_bulk_edges: bool = False) -> _RustworkxGraph:
        return self.rustworkx_from_local_graph(local_graph)

    def get_reader_endpoint(self) -> _RustworkxGraph:
        return self.graph

    def get_writer_endpoint(self) -> _RustworkxGraph:
        return self.graph

    def rustworkx_from_local_graph(self, local_graph: LocalGraph[_Block]) -> _RustworkxGraph:
        self.graph = PyDiGraph()
        vertices_to_add = []
        for index, vertex in enumerate(local_graph.vertices):
            attr = vertex.get_attribute_dict()
            vertices_to_add.append((index, attr))

        edges_to_add: list[tuple[int, int, dict[str, str | int]]] = [
            (
                e.origin,
                e.dest,
                {"label": e.label, "source": e.origin, "target": e.dest},
            )
            for e in local_graph.edges
        ]

        self.graph.add_nodes_from(vertices_to_add)
        self.graph.add_edges_from(edges_to_add)

        return self.graph

    def disconnect(self) -> None:
        # not used, but is an abstractmethod
        return None



from checkov.common.models.enums import CheckCategories
from checkov.kubernetes.checks.resource.base_spec_omitted_or_value_check import BaseSpecOmittedOrValueCheck


class ShareHostIPCPSP(BaseSpecOmittedOrValueCheck):
    def __init__(self) -> None:
        # CIS-1.3 1.7.3
        # CIS-1.5 5.2.3
        name = "Do not admit containers wishing to share the host IPC namespace"
        id = "CKV_K8S_3"
        # Location: PodSecurityPolicy.spec.hostIPC
        supported_kind = ("PodSecurityPolicy",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_inspected_key(self) -> str:
        return "spec/hostIPC"


check = ShareHostIPCPSP()



from __future__ import annotations

import itertools
import logging
from enum import Enum
from typing import Iterator, TYPE_CHECKING, Any

from checkov.common.bridgecrew.check_type import CheckType

if TYPE_CHECKING:
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report


class SecretsOmitterStatus(Enum):
    SUCCESS = 0
    INSUFFICIENT_REPORTS = 1


class SecretsOmitter:
    def __init__(self, reports: list[Report]):
        self.reports: list[Report] = [report for report in reports if report.check_type != CheckType.SECRETS]
        self.secrets_report: dict[str, Any] | None = self._get_secrets_report(reports)

    @staticmethod
    def _get_secrets_report(reports: list[Report]) -> dict[str, Any] | None:
        """
        Setting the secrets report from checkov runner or bucket
        """
        secrets_report_list = [report for report in reports if report.check_type == CheckType.SECRETS]
        return secrets_report_list[0].get_dict(full_report=True) if len(secrets_report_list) == 1 else None

    def _secret_check(self) -> Iterator[dict[str, Any]]:
        if not self.secrets_report:
            # Should not reach here, used for typing
            return

        for check in self.secrets_report.get("checks", {}).get("failed_checks"):
            yield check

    def _non_secret_check(self) -> Iterator[Record]:
        for report in self.reports:
            for check in itertools.chain(report.failed_checks, report.passed_checks):
                yield check

    @staticmethod
    def get_secret_lines(code_block: list[tuple[int, str]] | None) -> tuple[list[int], list[str]]:
        """
        Given a code block object, returns the lines containing asteriks including the line range
        :param code_block: list of tuples containing line number and the line itself
        :return: list of size 2, representing the range of lines containing secrets from code_block,
         and a list containing the lines from the range.
        """
        secret_lines_range = [-1, -1]
        secrets_lines: list[str] = []
        if not code_block:
            return secret_lines_range, secrets_lines

        for idx, line in code_block:
            if '*' in line:
                secrets_lines.append(line)
                if secret_lines_range[0] == -1:
                    secret_lines_range[0] = idx
                else:
                    secret_lines_range[1] = idx
        if secret_lines_range[1] == -1:
            secret_lines_range[1] = secret_lines_range[0]

        return secret_lines_range, secrets_lines

    @staticmethod
    def _line_range_overlaps(r1: list[int], r2: list[int]) -> bool:
        return r1[0] <= r2[1] and r1[1] >= r2[0]

    def omit(self) -> SecretsOmitterStatus:
        if not self.reports or not self.secrets_report:
            logging.debug("Insufficient reports to omit secrets")
            return SecretsOmitterStatus.INSUFFICIENT_REPORTS

        files_with_secrets: set[str] = {secret_check.get("file_path", "") for secret_check in self._secret_check()}
        for check in self._non_secret_check():
            check_file_path = check.file_path
            check_abs_file_path = check.file_abs_path
            check_line_range = check.file_line_range

            if (check_file_path not in files_with_secrets and check_abs_file_path not in files_with_secrets)\
                    or not check_line_range or None in check_line_range:
                continue

            for secret_check in self._secret_check():
                secret_check_file_path = secret_check.get("file_path", "")
                secret_check_line_range, secrets_check_lines = self.get_secret_lines(secret_check.get("code_block"))
                if secret_check_line_range == [-1, -1]:
                    continue

                if secret_check_file_path not in (check_file_path, check_abs_file_path) or \
                        not self._line_range_overlaps(secret_check_line_range, check_line_range):
                    continue

                if len(secrets_check_lines) != secret_check_line_range[1] - secret_check_line_range[0] + 1:
                    logging.error("Secrets lines does not match the length of the line range, sanity check failed")
                    continue

                for secret_line_index, omitted_line in \
                        zip(list(range(secret_check_line_range[0], secret_check_line_range[1] + 1)),  # noqa: B905
                            secrets_check_lines):
                    for entry_index, (line_index, _) in enumerate(check.code_block):
                        if secret_line_index == line_index:
                            check.code_block[entry_index] = (line_index, omitted_line)

        return SecretsOmitterStatus.SUCCESS



from checkov.common.models.consts import ANY_VALUE
from typing import Any
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AzureBatchAccountUsesKeyVaultEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Batch account uses key vault to encrypt data"
        id = "CKV_AZURE_76"
        supported_resources = ("Microsoft.Batch/batchAccounts",)
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def get_inspected_key(self) -> str:
        return "properties/keyVaultReference"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = AzureBatchAccountUsesKeyVaultEncryption()



import re

from checkov.common.sast.consts import SastLanguages

SUPPORTED_FILE_EXTENSIONS = [".tf", ".yml", ".yaml", ".json", ".template", ".bicep", ".hcl"]
SUPPORTED_PACKAGE_FILES = {
    "bower.json",
    "build.gradle",
    "build.gradle.kts",
    "go.sum",
    "gradle.properties",
    "METADATA",
    "npm-shrinkwrap.json",
    "package.json",
    "package-lock.json",
    "pom.xml",
    "requirements.txt"
}
SUPPORTED_FILES = SUPPORTED_PACKAGE_FILES.union({"Dockerfile"})

DEPENDENCY_TREE_SUPPORTED_FILES = {"yarn.lock", "Gemfile", "Gemfile.lock", "go.mod", "paket.dependencies", "paket.lock", "packages.config", "composer.json", "composer.lock"}

SCANNABLE_PACKAGE_FILES_EXTENSIONS = {".csproj"}

SCANNABLE_PACKAGE_FILES = SUPPORTED_PACKAGE_FILES | DEPENDENCY_TREE_SUPPORTED_FILES

SAST_SUPPORTED_FILE_EXTENSIONS = {
    SastLanguages.JAVA: ['.java'],
    SastLanguages.JAVASCRIPT: ['.js'],
    SastLanguages.TYPESCRIPT: ['.ts'],
    SastLanguages.PYTHON: ['.py'],
    SastLanguages.GOLANG: ['.go']
}

ANY_VALUE = "CKV_ANY"
DOCKER_IMAGE_REGEX = re.compile(r'(?:[^\s\/]+\/)?([^\s:]+):?([^\s]*)')
access_key_pattern = re.compile("(?<![A-Z0-9])[A-Z0-9]{20}(?![A-Z0-9])")  # nosec
secret_key_pattern = re.compile("(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{40}(?![A-Za-z0-9/+=])")  # nosec
linode_token_pattern = re.compile("(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{64}(?![A-Za-z0-9/+=])")  # nosec
bridgecrew_token_pattern = re.compile(r"^[a-f0-9]{8}-?[a-f0-9]{4}-?4[a-f0-9]{3}-?[89ab][a-f0-9]{3}-?[a-f0-9]{12}\Z")  # nosec
panos_api_key_pattern = re.compile(r"^LUFRPT1[a-zA-Z0-9]+==\Z")  # nosec
SLS_DEFAULT_VAR_PATTERN = re.compile(r"\${([^{}]+?)}")
YAML_COMMENT_MARK = '#'
TFC_HOST_NAME = "app.terraform.io"
ckv_check_id_pattern = re.compile(r"^CKV2?_[A-Za-z]*_[0-9]*$")



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GoogleStoragePublicAccessPrevention(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure public access prevention is enforced on Cloud Storage bucket"
        id = "CKV_GCP_114"
        supported_resources = ["google_storage_bucket"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "public_access_prevention"

    def get_expected_value(self) -> Any:
        return "enforced"


check = GoogleStoragePublicAccessPrevention()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EventHubNamespaceMinTLS12(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Event Hub Namespace uses at least TLS 1.2"
        id = "CKV_AZURE_223"
        supported_resources = ("azurerm_eventhub_namespace",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "minimum_tls_version"

    def get_expected_value(self) -> Any:
        return "1.2"


check = EventHubNamespaceMinTLS12()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CDNDisableHttpEndpoints(BaseResourceValueCheck):
    def __init__(self) -> None:
        # This is the full description of your check
        description = "Ensure the Azure CDN disables the HTTP endpoint"

        # This is the Unique ID for your check
        id = "CKV_AZURE_197"

        # These are the Terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = ("azurerm_cdn_endpoint",)

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "is_http_allowed"

    def get_expected_value(self) -> Any:
        return False


check = CDNDisableHttpEndpoints()



from __future__ import annotations

from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.foreach.foreach_entity_handler import ForeachEntityHandler

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph


class ForeachResourceHandler(ForeachEntityHandler):

    def __init__(self, local_graph: TerraformLocalGraph) -> None:
        super().__init__(local_graph, BlockType.RESOURCE)



from typing import Dict, Any, List

import pytest

from cdk_integration_tests.utils import run_check, load_failed_checks_from_file

LANGUAGE = 'typescript'


@pytest.fixture(scope="session", autouse=True)
def failed_checks() -> Dict[str, List[Dict[str, Any]]]:
    report_failed_checks = load_failed_checks_from_file(LANGUAGE)
    yield report_failed_checks


def test_CKV_AWS_131_ALBDropHttpHeaders(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_131", policy_name="ALBDropHttpHeaders",
              language="typescript")


def test_CKV_AWS_2_ALBListenerHTTPS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_2", policy_name="ALBListenerHTTPS", language="typescript")


def test_CKV_AWS_59_APIGatewayAuthorization(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_59", policy_name="APIGatewayAuthorization",
              language="typescript")


def test_CKV_AWS_76_APIGatewayAccessLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_76", policy_name="APIGatewayAccessLogging",
              language="typescript")


def test_CKV_AWS_120_APIGatewayCacheEnable(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_120", policy_name="APIGatewayCacheEnable",
              language="typescript")


def test_CKV_AWS_95_APIGatewayV2AccessLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_95", policy_name="APIGatewayV2AccessLogging",
              language="typescript")


def test_CKV_AWS_73_APIGatewayXray(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_73", policy_name="APIGatewayXray", language="typescript")


def test_CKV_AWS_194_AppSyncFieldLevelLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_194", policy_name="AppSyncFieldLevelLogs",
              language="typescript")


def test_CKV_AWS_193_AppSyncLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_193", policy_name="AppSyncLogging", language="typescript")


def test_CKV_AWS_82_AthenaWorkgroupConfiguration(failed_checks):
    # need to wait for variable rendering in TS
    run_check(check_results=failed_checks, check_id="CKV_AWS_82", policy_name="AthenaWorkgroupConfiguration",
              language="typescript")


def test_CKV_AWS_131_AmazonMQBrokerPublicAccess(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_69", policy_name="AmazonMQBrokerPublicAccess",
              language="typescript")


def test_CKV_AWS_96_AuroraEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_96", policy_name="AuroraEncryption",
              language="typescript")


def test_CKV_AWS_166_BackupVaultEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_166", policy_name="BackupVaultEncrypted",
              language="typescript")


def test_CKV_AWS_174_CloudFrontTLS12(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_174", policy_name="CloudFrontTLS12", language="typescript")


def test_CKV_AWS_36_CloudTrailLogValidation(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_36", policy_name="CloudTrailLogValidation",
              language="typescript")


def test_CKV_AWS_20_S3PublicACLRead(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_20", policy_name="S3PublicACLRead", language="typescript")


def test_CKV_AWS_56_S3RestrictPublicBuckets(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_56", policy_name="S3RestrictPublicBuckets",
              language="typescript")


def test_CKV_AWS_149_SecretManagerSecretEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_149", policy_name="SecretManagerSecretEncrypted",
              language="typescript")


def test_CKV_AWS_23_SecurityGroupRuleDescription(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_23", policy_name="SecurityGroupRuleDescription",
              language="typescript")


def test_CKV_AWS_26_SNSTopicEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_26", policy_name="SNSTopicEncryption",
              language="typescript")


def test_CKV_AWS_27_SQSQueueEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_27", policy_name="SQSQueueEncryption",
              language="typescript")


def test_CKV_AWS_164_TransferServerIsPublic(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_164", policy_name="TransferServerIsPublic",
              language="typescript")


def test_CKV_AWS_123_VPCEndpointAcceptanceConfigured(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_123", policy_name="VPCEndpointAcceptanceConfigured",
              language="typescript")


def test_CKV_AWS_68_WAFEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_68", policy_name="WAFEnabled", language="typescript")


def test_CKV_AWS_156_WorkspaceRootVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_156", policy_name="WorkspaceRootVolumeEncrypted",
              language="typescript")


def test_CKV_AWS_155_WorkspaceUserVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_155", policy_name="WorkspaceUserVolumeEncrypted",
              language="typescript")


def test_CKV_AWS_88_EC2PublicIP(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_88", policy_name="EC2PublicIP", language="typescript")


def test_CKV_AWS_163_ECRImageScanning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_163", policy_name="ECRImageScanning",
              language="typescript")


def test_CKV_AWS_51_ECRImmutableTags(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_51", policy_name="ECRImmutableTags", language="typescript")


def test_CKV_AWS_136_ECRRepositoryEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_136", policy_name="ECRRepositoryEncrypted",
              language="typescript")


def test_CKV_AWS_65_ECSClusterContainerInsights(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_65", policy_name="ECSClusterContainerInsights",
              language="typescript")


def test_CKV_AWS_97_ECSTaskDefinitionEFSVolumeEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_97", policy_name="ECSTaskDefinitionEFSVolumeEncryption",
              language="typescript")


def test_CKV_AWS_42_EFSEncryptionEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_42", policy_name="EFSEncryptionEnabled",
              language="typescript")


def test_CKV_AWS_58_EKSSecretsEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_58", policy_name="EKSSecretsEncryption",
              language="typescript")


def test_CKV_AWS_29_ElasticacheReplicationGroupEncryptionAtRest(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_29",
              policy_name="ElasticacheReplicationGroupEncryptionAtRest", language="typescript")


def test_CKV_AWS_30_ElasticacheReplicationGroupEncryptionAtTransit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_30",
              policy_name="ElasticacheReplicationGroupEncryptionAtTransit",
              language="typescript")


def test_CKV_AWS_31_ElasticacheReplicationGroupEncryptionAtTransitAuthToken(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_31",
              policy_name="ElasticacheReplicationGroupEncryptionAtTransitAuthToken",
              language="typescript")


def test_CKV_AWS_83_ElasticsearchDomainEnforceHTTPS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_83", policy_name="ElasticsearchDomainEnforceHTTPS",
              language="typescript")


def test_CKV_AWS_84_ElasticsearchDomainLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_84", policy_name="ElasticsearchDomainLogging",
              language="typescript")


def test_CKV_AWS_92_ELBAccessLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_92", policy_name="ELBAccessLogs", language="typescript")


def test_CKV_AWS_91_ELBv2AccessLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_91", policy_name="ELBv2AccessLogs", language="typescript")


def test_CKV_AWS_158_CloudWatchLogGroupKMSKey(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_158", policy_name="CloudWatchLogGroupKMSKey",
              language="typescript")


def test_CKV_AWS_66_CloudWatchLogGroupRetention(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_66", policy_name="CloudWatchLogGroupRetention",
              language="typescript")


def test_CKV_AWS_34_CloudfrontDistributionEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_34", policy_name="CloudfrontDistributionEncryption",
              language="typescript")


def test_CKV_AWS_86_CloudfrontDistributionLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_86", policy_name="CloudfrontDistributionLogging",
              language="typescript")


def test_CKV_AWS_35_CloudtrailEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_35", policy_name="CloudtrailEncryption",
              language="typescript")


def test_CKV_AWS_67_CloudtrailMultiRegion(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_67", policy_name="CloudtrailMultiRegion",
              language="typescript")


def test_CKV_AWS_78_CodeBuildProjectEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_78", policy_name="CodeBuildProjectEncryption",
              language="typescript")


def test_CKV_AWS_47_DAXEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_47", policy_name="DAXEncryption", language="typescript")


def test_CKV_AWS_89_DMSReplicationInstancePubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_89",
              policy_name="DMSReplicationInstancePubliclyAccessible", language="typescript")


def test_CKV_AWS_104_DocDBAuditLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_104", policy_name="DocDBAuditLogs", language="typescript")


def test_CKV_AWS_74_DocDBEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_74", policy_name="DocDBEncryption", language="typescript")


def test_CKV_AWS_90_DocDBTLS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_90", policy_name="DocDBTLS", language="typescript")


def test_CKV_AWS_165_DynamodbGlobalTableRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_165", policy_name="DynamodbGlobalTableRecovery",
              language="typescript")


def test_CKV_AWS_28_DynamodbRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_28", policy_name="DynamodbRecovery", language="typescript")


def test_CKV_AWS_3_EBSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_3", policy_name="EBSEncryption", language="typescript")


def CKV_AWS_377_S3BucketLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_377", policy_name="S3BucketLogging",
              language="typescript")


def CKV_AWS_375_S3BucketEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_375", policy_name="S3BucketEncryption",
              language="typescript")


def CKV_AWS_379_S3BucketVersioning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_379", policy_name="S3BucketVersioning",
              language="typescript")


def CKV_AWS_376_S3BucketKMSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_376", policy_name="S3BucketKMSEncryption",
              language="typescript")


def CKV_AWS_378_S3BucketPublicAccessBlock(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_378", policy_name="S3BucketPublicAccessBlock",
              language="typescript")


def test_CKV_AWS_195_GlueSecurityConfigurationEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_195", policy_name="GlueSecurityConfigurationEnabled",
              language="typescript")


def CKV_AWS_5_ElasticsearchEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_5", policy_name="ElasticsearchEncryption",
              language="typescript")


def CKV_AWS_6_ElasticsearchNodeToNodeEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_6", policy_name="ElasticsearchNodeToNodeEncryption",
              language="typescript")


def CKV_AWS_94_GlueDataCatalogEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_94", policy_name="GlueDataCatalogEncryption",
              language="typescript")


def CKV_AWS_99_GlueSecurityConfiguration(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_99", policy_name="GlueSecurityConfiguration",
              language="typescript")


def CKV_AWS_195_GlueSecurityConfigurationEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_195", policy_name="GlueSecurityConfigurationEnabled",
              language="typescript")


def CKV_AWS_40_IAMPolicyAttachedToGroupOrRoles(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_40", policy_name="IAMPolicyAttachedToGroupOrRoles",
              language="typescript")


def CKV_AWS_43_KinesisStreamEncryptionType(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_43", policy_name="KinesisStreamEncryptionType",
              language="typescript")


def CKV_AWS_116_LambdaDLQConfigured(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_116", policy_name="LambdaDLQConfigured",
              language="typescript")


def CKV_AWS_45_LambdaEnvironmentCredentials(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_45", policy_name="LambdaEnvironmentCredentials",
              language="typescript")


def CKV_AWS_173_LambdaEnvironmentEncryptionSettings(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_173", policy_name="LambdaEnvironmentEncryptionSettings",
              language="typescript")


def CKV_AWS_115_LambdaFunctionLevelConcurrentExecutionLimit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_115",
              policy_name="LambdaFunctionLevelConcurrentExecutionLimit", language="typescript")


def CKV_AWS_117_LambdaInVPC(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_117", policy_name="LambdaInVPC", language="typescript")


def CKV_AWS_8_LaunchConfigurationEBSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_8", policy_name="LaunchConfigurationEBSEncryption",
              language="typescript")


def CKV_AWS_44_NeptuneClusterStorageEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_44", policy_name="NeptuneClusterStorageEncrypted",
              language="typescript")


def CKV_AWS_118_RDSEnhancedMonitorEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_118", policy_name="RDSEnhancedMonitorEnabled",
              language="typescript")


def CKV_AWS_366_RDSMultiAZEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_366", policy_name="RDSMultiAZEnabled",
              language="typescript")


def CKV_AWS_367_RDSPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_367", policy_name="RDSPubliclyAccessible",
              language="typescript")


def CKV_AWS_369_RedShiftSSL(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_369", policy_name="RedShiftSSL",
              language="typescript")


def CKV_AWS_368_RedshiftClusterEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_368", policy_name="RedshiftClusterEncryption",
              language="typescript")


def CKV_AWS_370_RedshiftClusterLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_370", policy_name="RedshiftClusterLogging",
              language="typescript")


def CKV_AWS_371_RedshiftClusterPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_371", policy_name="RedshiftClusterPubliclyAccessible",
              language="typescript")


def CKV_AWS_372_RedshiftInEc2ClassicMode(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_372", policy_name="RedshiftInEc2ClassicMode",
              language="typescript")


def CKV_AWS_373_S3BlockPublicACLs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_373", policy_name="S3BlockPublicACLs",
              language="typescript")


def CKV_AWS_374_S3BlockPublicPolicy(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_374", policy_name="S3BlockPublicPolicy",
              language="typescript")



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class ACRAnonymousPullDisabled(BaseResourceCheck):
    ANONYMOUS_PULL_SKUS = {"Standard", "Premium"}  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        name = "Ensures that ACR disables anonymous pulling of images"
        id = "CKV_AZURE_138"
        supported_resources = ("Microsoft.ContainerRegistry/registries",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties", {})

        anonymousPullEnabled = properties.get("anonymousPullEnabled")

        sku = conf.get("sku")

        if (
                sku is not None
                and isinstance(sku.get("name"), str)
                and sku.get("name") in ACRAnonymousPullDisabled.ANONYMOUS_PULL_SKUS
                and properties
                and anonymousPullEnabled
        ):
            return CheckResult.FAILED

        return CheckResult.PASSED


check = ACRAnonymousPullDisabled()



from checkov.terraform.checks.resource.aws.AbsSecurityGroupUnrestrictedIngress import\
    AbsSecurityGroupUnrestrictedIngress


class SecurityGroupUnrestrictedIngressAll(AbsSecurityGroupUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_277", port=-1)


check = SecurityGroupUnrestrictedIngressAll()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AppsyncCacheEncryptionAtRest(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure AppSync API Cache is encrypted at rest"
        id = "CKV_AWS_214"
        supported_resources = ["aws_appsync_api_cache"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "at_rest_encryption_enabled"


check = AppsyncCacheEncryptionAtRest()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck

SUPPORTED_ENGINES = {"aurora", "aurora-mysql"}


class RDSClusterAuroraBacktrack(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CP-10, NIST.800-53.r5 CP-6, NIST.800-53.r5 CP-6(1), NIST.800-53.r5 CP-6(2), NIST.800-53.r5 CP-9,
        NIST.800-53.r5 SI-13(5)	Amazon Aurora clusters should have backtracking enabled
        """
        name = "Ensure that RDS Aurora Clusters have backtracking enabled"
        id = "CKV_AWS_326"
        supported_resources = ("aws_rds_cluster",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_attribute_result=CheckResult.FAILED,
        )

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        engine = conf.get("engine")
        if engine and isinstance(engine, list) and engine[0] not in SUPPORTED_ENGINES:
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf=conf)

    def get_inspected_key(self) -> str:
        return "backtrack_window"

    def get_forbidden_values(self) -> list[Any]:
        return [0]


check = RDSClusterAuroraBacktrack()



import shutil
import sys
from io import StringIO
from typing import Dict


class LoggerStreams:
    def __init__(self) -> None:
        self._streams: Dict[str, StringIO] = {}

    def add_stream(self, name: str, stream: StringIO) -> None:
        self._streams[name] = stream

    def get_streams(self) -> Dict[str, StringIO]:
        return self._streams

    def print_to_files(self) -> None:
        for key, value in self._streams.items():
            with open(f'./checkov_debug_{key}.log', 'w') as fp:
                value.seek(0)
                shutil.copyfileobj(value, fp)

    def print_to_console(self) -> None:
        for key, value in self._streams.items():
            print('----')
            print(f'Logger of {key} start')
            print(value.getvalue(), file=sys.stderr)


logger_streams = LoggerStreams()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class PeerClientCertAuthTrue(BaseK8Check):
    def __init__(self) -> None:
        name = "Ensure that the --peer-client-cert-auth argument is set to true"
        id = "CKV_K8S_121"
        supported_kind = ("Pod",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        if conf.get("metadata", {}).get("name") == "etcd":
            containers = conf.get("spec", {}).get("containers")
            if containers:
                for container in containers:
                    if container.get("args") is not None:
                        if "--peer-client-cert-auth=true" not in container["args"]:
                            return CheckResult.FAILED
                return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = PeerClientCertAuthTrue()



from __future__ import annotations
from typing import Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck

BAD_CIPHERS = {
    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
    "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
    "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
    "TLS_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_RSA_WITH_AES_256_CBC_SHA256",
    "TLS_RSA_WITH_AES_128_CBC_SHA256",
    "TLS_RSA_WITH_AES_256_CBC_SHA",
    "TLS_RSA_WITH_AES_128_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384 ",
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
    "TLS_DHE_DSS_WITH_AES_256_CBC_SHA256",
    "TLS_DHE_DSS_WITH_AES_128_CBC_SHA256 ",
    "TLS_DHE_DSS_WITH_AES_256_CBC_SHA",
    "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",
    "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
    "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA",
}
PROTOCOL_VERSIONS = {"TLSv1_2", "TLSv1_3"}


class AppGWDefinesSecureProtocols(BaseResourceCheck):
    def __init__(self) -> None:
        """
        https://azure.github.io/PSRule.Rules.Azure/en/rules/Azure.AppGw.SSLPolicy/
        """
        name = "Ensure Application Gateway defines secure protocols for in transit communication"
        id = "CKV_AZURE_218"
        supported_resources = ("Microsoft.Network/applicationGateways",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name,
                         id=id,
                         categories=categories,
                         supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        sslPolicy = conf["properties"].get("sslPolicy")
        if sslPolicy and isinstance(sslPolicy, dict):
            policyType = sslPolicy.get("policyType")
            if policyType != "Predefined":
                protocolversion = sslPolicy.get("minProtocolVersion")
                if (
                        protocolversion and isinstance(protocolversion, str)
                        and protocolversion in PROTOCOL_VERSIONS
                ):
                    ciphers = sslPolicy.get("cipherSuites")
                    if ciphers and isinstance(ciphers, list) and any(cipher in BAD_CIPHERS for cipher in ciphers):
                        return CheckResult.FAILED
                    return CheckResult.PASSED

            policyName = sslPolicy.get("policyName")
            if policyName == "AppGwSslPolicy20220101S":
                return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED


check = AppGWDefinesSecureProtocols()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class MQBrokerAuditLogging(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure MQ Broker Audit logging is enabled"
        id = "CKV_AWS_197"
        supported_resources = ("aws_mq_broker",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/security-logging-monitoring-rabbitmq.html
        # Audit logging is not supported for RabbitMQ brokers.
        if conf.get("engine_type") == ["RabbitMQ"]:
            return CheckResult.UNKNOWN
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "logs/[0]/audit"


check = MQBrokerAuditLogging()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class PubsubSpecifyIdentity(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Web PubSub uses managed identities to access Azure resources"
        id = "CKV_AZURE_176"
        supported_resources = ("azurerm_web_pubsub",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "identity/[0]/type"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = PubsubSpecifyIdentity()



from __future__ import annotations

from abc import abstractmethod
from typing import List, Any, Tuple, Dict, TYPE_CHECKING, Optional

from networkx import DiGraph

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

if TYPE_CHECKING:
    from checkov.common.typing import LibraryGraph


class BaseComplexSolver(BaseSolver):
    operator = ""  # noqa: CCE003  # a static attribute

    def __init__(self, solvers: List[BaseSolver], resource_types: List[str]) -> None:
        if solvers is None:
            solvers = []
        self.solvers = solvers
        self.resource_types = resource_types
        super().__init__(SolverType.COMPLEX)

    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError()

    def _get_negative_op(self, *args: Any) -> Any:
        return not self._get_operation(args)

    @abstractmethod
    def get_operation(self, vertex: Dict[str, Any]) -> Optional[bool]:
        raise NotImplementedError()

    def run(self, graph_connector: LibraryGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        passed_vertices = []
        failed_vertices = []
        unknown_vertices = []

        if isinstance(graph_connector, DiGraph):
            for _, data in graph_connector.nodes(data=True):
                if self.resource_type_pred(data, self.resource_types):
                    result = self.get_operation(data)
                    if result is None:
                        unknown_vertices.append(data)
                    elif result:
                        passed_vertices.append(data)
                    else:
                        failed_vertices.append(data)
            return passed_vertices, failed_vertices, unknown_vertices

        # isinstance(graph_connector, PyDiGraph):
        for _, data in graph_connector.nodes():
            if self.resource_type_pred(data, self.resource_types):
                result = self.get_operation(data)
                if result is None:
                    unknown_vertices.append(data)
                elif result:
                    passed_vertices.append(data)
                else:
                    failed_vertices.append(data)
        return passed_vertices, failed_vertices, unknown_vertices



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EventgridDomainNetworkAccess(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Azure Event Grid Domain public network access is disabled"
        id = "CKV_AZURE_106"
        supported_resources = ['azurerm_eventgrid_domain']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources, )

    def get_inspected_key(self):
        return 'public_network_access_enabled'

    def get_expected_value(self):
        return False


check = EventgridDomainNetworkAccess()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import List, Dict, Any

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.module.registry import module_registry


class BaseModuleCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_resources: Iterable[str] | None = None,
        guideline: str | None = None
    ) -> None:
        """
        Base class for terraform module call related checks.

        :param name: an error message that is shown, when the check failed.
        :param id: the id of the check
        :param categories: categories of the check
        :param supported_resources: DEPRECATED the resources that this check applies to.

            This is deprecated because there is only one resource type that is valid for
            checks that extend this class.
        """
        if supported_resources is None:
            supported_resources = ["module"]
        super().__init__(
            name=name, id=id, categories=categories, supported_entities=supported_resources,
            block_type="module", guideline=guideline
        )
        self.supported_resources = supported_resources
        module_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, List[Any]], entity_type: str) -> CheckResult:
        # entity_type is always 'module'
        return self.scan_module_conf(conf)

    @abstractmethod
    def scan_module_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        raise NotImplementedError()

    @staticmethod
    def is_git_source(source: str) -> bool:
        return source.startswith('git@') or source.startswith('git::') or source.startswith('github.com') or source.startswith('bitbucket.org')



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.data_structures_utils import find_in_dict


class DatabricksWorkspaceDBFSRootEncryptedWithCustomerManagedKey(BaseResourceCheck):
    def __init__(self) -> None:
        # https://learn.microsoft.com/en-us/azure/templates/microsoft.databricks/workspaces?pivots=deployment-language-arm-template#workspaceencryptionparameter-1
        name = "Ensure that Databricks Workspaces enables customer-managed key for root DBFS encryption"
        id = "CKV2_AZURE_48"
        supported_resources = ("Microsoft.Databricks/workspaces",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        parameters = conf.get("properties", {}).get("parameters")
        prepare_encryption = find_in_dict(input_dict=parameters, key_path="prepareEncryption/value")
        if not prepare_encryption or str(prepare_encryption).lower() != "true":
            return CheckResult.FAILED

        encryption_settings = find_in_dict(input_dict=parameters, key_path="encryption/value")
        if not encryption_settings:
            return CheckResult.FAILED

        return CheckResult.PASSED


check = DatabricksWorkspaceDBFSRootEncryptedWithCustomerManagedKey()



from typing import List, Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudArmorWAFACLCVE202144228(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Cloud Armor prevents message lookup in Log4j2. See CVE-2021-44228 aka log4jshell"
        id = "CKV_GCP_73"
        supported_resources = ("google_compute_security_policy",)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        self.evaluated_keys = ["rule"]
        rules = conf.get("rule") or []
        for idx_rule, rule in enumerate(force_list(rules)):
            self.evaluated_keys = [
                f"rule/[{idx_rule}]/action",
                f"rule/[{idx_rule}]/preview",
                f"rule/[{idx_rule}]/match/[0]/expr/[0]/expression",
            ]
            match = rule.get("match")
            if match and isinstance(match, list):
                expr = match[0].get("expr")
                if expr and isinstance(expr[0], dict):
                    if expr[0].get("expression") == ["evaluatePreconfiguredExpr('cve-canary')"]:
                        if rule.get("action") == ["allow"]:
                            return CheckResult.FAILED
                        if rule.get("preview") == [True]:
                            return CheckResult.FAILED
                        return CheckResult.PASSED
                    elif expr[0].get("expression") == ["evaluatePreconfiguredWaf('cve-canary')"]:
                        if rule.get("action") == ["allow"]:
                            return CheckResult.FAILED
                        if rule.get("preview") == [True]:
                            return CheckResult.FAILED
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = CloudArmorWAFACLCVE202144228()



from typing import List, Dict, Any, Tuple, Optional, Union

from termcolor import colored

from checkov.common.bridgecrew.severities import Severity
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult
from checkov.common.sast.report_types import MatchMetadata, MatchLocation


class SastRecord(Record):
    def __init__(self,
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 severity: Optional[Severity],
                 metadata: Optional[MatchMetadata] = None,
                 bc_check_id: Optional[str] = None,
                 cwe: Optional[Union[List[str], str]] = None,
                 owasp: Optional[Union[List[str], str]] = None,
                 show_severity: Optional[bool] = False  # should be false in case the severities are just a default value
                 ) -> None:
        super().__init__(
            check_id=check_id,
            bc_check_id=bc_check_id,
            check_name=check_name,
            check_result=check_result,
            code_block=code_block,
            file_path=file_path,
            file_line_range=file_line_range,
            resource=resource,
            evaluations=evaluations,
            check_class=check_class,
            file_abs_path=file_abs_path,
            severity=severity,
        )
        self.cwe = cwe
        self.owasp = owasp
        self.show_severity = show_severity
        self.metadata = metadata

    def to_string(self, compact: bool = False, use_bc_ids: bool = False) -> str:
        status = ""
        status_color = "white"
        suppress_comment = ""
        if self.check_result["result"] == CheckResult.PASSED:
            status = CheckResult.PASSED.name
            status_color = "green"
        elif self.check_result["result"] == CheckResult.FAILED:
            status = CheckResult.FAILED.name
            status_color = "red"
        elif self.check_result["result"] == CheckResult.SKIPPED:
            status = CheckResult.SKIPPED.name
            status_color = "blue"
            suppress_comment = "\tSuppress comment: {}\n".format(self.check_result.get("suppress_comment", ""))

        check_message = colored('Check: {}: "{}"\n'.format(self.get_output_id(use_bc_ids), self.check_name),
                                "white")
        guideline_message = self.get_guideline_string(self.guideline)

        severity_message = f'\tSeverity: {self.severity.name}\n' if self.severity and self.show_severity else ''

        evaluation_message = self.get_evaluation_string(self.evaluations, self.code_block)

        cwe_message = colored(f'\t{self.cwe}\n') if self.cwe else ''

        if self.metadata and self.metadata.taint_mode and self.metadata.taint_mode.data_flow and \
                len(self.metadata.taint_mode.data_flow) > 0:
            code_lines, file_details = self.get_code_lines_taint(self.metadata.taint_mode.data_flow)
        else:
            file_details = f'{self.file_path}:{" -> ".join([str(x) for x in self.file_line_range])}' if \
                self.file_line_range[0] != self.file_line_range[-1] else \
                f'{self.file_path}:{str(self.file_line_range[0])}'
            code_lines = self.get_code_lines_string(self.code_block)

        detail = self.get_details_string(self.details)
        caller_file_details = self.get_caller_file_details_string(self.caller_file_path, self.caller_file_line_range)
        status_message = colored("\t{} for file - {}\n".format(status, file_details), status_color)

        if self.check_result["result"] == CheckResult.FAILED and code_lines and not compact:
            return f"{check_message}{severity_message}{status_message}{cwe_message}{detail}{caller_file_details}{guideline_message}{code_lines}{evaluation_message}\n"

        if self.check_result["result"] == CheckResult.SKIPPED:
            return f"{check_message}{severity_message}{status_message}{cwe_message}{suppress_comment}{detail}{caller_file_details}{guideline_message}\n"
        else:
            return f"{check_message}{severity_message}{status_message}{cwe_message}{detail}{caller_file_details}{evaluation_message}{guideline_message}\n"

    def get_code_lines_taint(self, dataflows: List[MatchLocation]) -> Tuple[str, str]:
        code_lines = ""
        last_file = dataflows[0].path.split('/')[-1]
        last_line_num = dataflows[0].start.row
        code_lines += colored("\t\t" + last_file, 'light_yellow')
        file_details = last_file
        for df in dataflows:
            cur_file = df.path.split('/')[-1]
            cur_line_num = df.start.row
            if cur_file != last_file:
                code_lines += colored("\t\t" + cur_file, 'light_yellow')
                file_details += "->" + cur_file
                last_file = cur_file
            else:
                if cur_line_num != last_line_num and cur_line_num != last_line_num + 1:
                    code_lines += colored("\t\t...", 'light_yellow')
            code_lines += self.get_code_lines_string([(cur_line_num, df.code_block)])
            file_details += "->" + str(cur_line_num)
            last_line_num = cur_line_num
        return code_lines, file_details



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AKSOnlyCriticalPodsOnSystemNodes(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        Microsoft recommends to isolate critical system pods from application pods
        to prevent misconfigured or rogue application pods from accidentally killing system pods.

        This can be enforced by creating a dedicated system node pool with the CriticalAddonsOnly=true:NoSchedule taint
        to prevent application pods from being scheduled on system node pools.
        """
        name = "Ensure that only critical system pods run on system nodes"
        id = "CKV_AZURE_232"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "default_node_pool/[0]/only_critical_addons_enabled"


check = AKSOnlyCriticalPodsOnSystemNodes()



from typing import List, Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class FunctionAppDisallowCORS(BaseResourceNegativeValueCheck):

    def __init__(self) -> None:
        name = "Ensure function apps are not accessible from all regions"
        id = "CKV_AZURE_62"
        supported_resources = ("Microsoft.Web/sites",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories,
                         supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "properties/siteConfig/cors/allowedOrigins"

    def get_forbidden_values(self) -> List[Any]:
        return ["*"]


check = FunctionAppDisallowCORS()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class BigQueryTableDeletionProtection(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure BigQuery tables have deletion protection enabled"
        id = "CKV_GCP_121"
        supported_resources = ['google_bigquery_table']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return 'deletion_protection'

    def get_expected_value(self) -> bool:
        return True


check = BigQueryTableDeletionProtection()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ECSClusterLoggingEncryptedWithCMK(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure ECS Cluster logging uses CMK"
        id = "CKV_AWS_224"
        supported_resources = ("aws_ecs_cluster",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        configuration = conf.get("configuration")
        if configuration and isinstance(configuration, list) and isinstance(configuration[0], dict):
            execute_command = configuration[0].get("execute_command_configuration")
            if execute_command and isinstance(execute_command, list):
                execute_command = execute_command[0]
                if isinstance(execute_command, dict) and not execute_command.get("logging") == ["NONE"]:
                    if execute_command.get("kms_key_id"):
                        log_conf = execute_command.get("log_configuration")
                        if log_conf and isinstance(log_conf, list):
                            log_conf = log_conf[0]
                            if isinstance(log_conf, dict) and (
                                log_conf.get("cloud_watch_encryption_enabled") == [True]
                                or log_conf.get("s3_bucket_encryption_enabled") == [True]
                            ):
                                return CheckResult.PASSED

                    return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = ECSClusterLoggingEncryptedWithCMK()
